'use strict';

/* Services */


// Demonstrate how to register services
// In this case it is a simple value service.
angular.module('myApp.services', [])
    .factory('prayerService',['$rootScope','geolocation','ngGPlacesAPI',function($rootScope,geolocation,ngGPlacesAPI){
        return {
            //get current location
            getLocation: function(){
                return geolocation.getLocation().then(function(data){
                    return data;
                });
            },

            //get prayer times
            getPrayerTimes: function(){
                return this.getLocation().then(function(data){
                    return prayTimes.getTimes(new Date(), [data.coords.latitude, data.coords.longitude]);
                });
            },

            //convert time string formatted as 00:00 to minutes
            getPrayerTimeInMin: function(timeString){
                var splitString     = timeString.split(':');
                var prayerTimeInMin = parseInt(splitString[0]) * 60 + parseInt(splitString[1]);
                return prayerTimeInMin;
            },

            // get prayer times and countdown to next prayer
            getPrayerCountDown:function(){
                var self = this;
                return this.getPrayerTimes().then(function(data){
                    var currentTimeInMin = new Date().getHours() * 60 + new Date().getMinutes();
                    var timeRemaining;
                    var messageEn;
                    var messageAr;

                    //if current time after fajr and before sunrise
                    // show time remaining till dhurh etc...

                    if(currentTimeInMin < self.getPrayerTimeInMin(data.sunrise) &&
                        currentTimeInMin > self.getPrayerTimeInMin(data.fajr)){
                        timeRemaining = self.getPrayerTimeInMin(data.sunrise) - currentTimeInMin;
                        messageEn = "Time remaining for Sunrise";
                        messageAr = "الوقت المتبقي على الشروق";

                    }else if(currentTimeInMin > self.getPrayerTimeInMin(data.sunrise)&&
                        currentTimeInMin < self.getPrayerTimeInMin(data.dhuhr)){
                        timeRemaining = self.getPrayerTimeInMin(data.dhuhr) - currentTimeInMin;
                        messageEn = "Time remaining for Dhuhr";
                        messageAr = "الوقت المتبقي على أذان الظهر";

                    }else if(currentTimeInMin > self.getPrayerTimeInMin(data.dhuhr)&&
                        currentTimeInMin < self.getPrayerTimeInMin(data.asr)){
                        timeRemaining = self.getPrayerTimeInMin(data.asr) - currentTimeInMin;
                        messageEn = "Time remaining for Asr";
                        messageAr = "الوقت المتبقي على أذان العصر";


                    }else if(currentTimeInMin > self.getPrayerTimeInMin(data.asr)&&
                        currentTimeInMin < self.getPrayerTimeInMin(data.maghrib)){
                        timeRemaining = self.getPrayerTimeInMin(data.maghrib) - currentTimeInMin;
                        messageEn = "Time remaining for Maghrib";
                        messageAr = "الوقت المتبقي على أذان المغرب";


                    }else if(currentTimeInMin > self.getPrayerTimeInMin(data.maghrib)&&
                        currentTimeInMin < self.getPrayerTimeInMin(data.isha)){
                        timeRemaining = self.getPrayerTimeInMin(data.isha) - currentTimeInMin;
                        messageEn = "Time remaining for Isha";
                        messageAr = "الوقت المتبقي على أذان العشاء";

                    }else if(currentTimeInMin > self.getPrayerTimeInMin(data.isha)&&
                        currentTimeInMin < (24*60)){
                        timeRemaining = self.getPrayerTimeInMin(data.fajr) + ((24*60) - currentTimeInMin);
                        messageEn = "Time remaining for Fajr";
                        messageAr = "الوقت المتبقي على أذان الفجر";

                    }else if(currentTimeInMin < self.getPrayerTimeInMin(data.fajr)&&
                        currentTimeInMin > 0){
                        timeRemaining = self.getPrayerTimeInMin(data.fajr) - currentTimeInMin;
                        messageEn = "Time remaining for Fajr";
                        messageAr = "الوقت المتبقي على أذان الفجر";

                    }

                    return {
                        timeRemaining  : timeRemaining,
                        messageEn      : messageEn,
                        messageAr      : messageAr,
                        prayerTimes    : data
                    };
                });
            },
            // get nearest mosque using google places api
            getNearestMosque: function(){
                return this.getLocation().then(function(data){
                    /*var options = {
                        radius: 500,
                        latitude:data.coords.latitude,
                        longitude:data.coords.longitude,
                        sensor: false,
                        types: ['mosque'],
                        map:map,
                        nearbySearchKeys: ['name', 'reference', 'vicinity'],
                        placeDetailsKeys: ['formatted_address', 'formatted_phone_number',
                            'reference', 'website'
                        ],
                        nearbySearchErr: 'Unable to find nearby places',
                        placeDetailsErr: 'Unable to find place details'
                    };
                    return ngGPlacesAPI.nearbySearch(options)
                           .then(function(data){
                            console.log(data);
                             return data;
                    });*/
                    var lat  = data.coords.latitude;
                    var long = data.coords.longitude;

                    google.maps.event.addDomListener(window, 'load', initialize(lat,long));

                    function initialize(lat,long) {
                        var myLocation = new google.maps.LatLng(lat, long);

                        var map = new google.maps.Map(document.getElementById('map'), {
                            center: myLocation,
                            zoom: 15,
                            scrollwheel: false
                        });

                        // Specify location, radius and place types for your Places API search.
                        var request = {
                            location: myLocation,
                            radius: '500',
                            types: ['mosque']
                        };

                        // Create the PlaceService and send the request.
                        // Handle the callback with an anonymous function.
                        var service = new google.maps.places.PlacesService(map);

                        service.nearbySearch(request, function(results, status) {
                            if (status == google.maps.places.PlacesServiceStatus.OK) {
                                for (var i = 0; i < results.length; i++) {
                                    var place = results[i];
                                    // If the request succeeds, draw the place location on
                                    // the map as a marker, and register an event to handle a
                                    // click on the marker.
                                    var marker = new google.maps.Marker({
                                        map: map,
                                        position: place.geometry.location
                                    });
                                }
                            }
                        });
                    }
                });
            },
            getCountDown: function(){

            }
        };
    }])
    .value('version', '0.1');
